// jQuery-based script for HelpSmith
// Implements the possibility to add collapsible "toggle" sections in HTML Help and Web Help documentation
// Copyright © 2017 http://www.HelpSmith.com

function toggle() {
	var self = this;

	this.init = function() {
		// prepare document
		self.process_toggles();
		
		// update text for "toggle all" links
		self.update_toggle_all_link_text();
		
		// update document
		this.update_document();
		
		// set event handlers
		$(document).on('click', '.toggle_link', self.toggle_link_click_handler);
		$(document).on('click', '.toggle_all', self.toggle_all_click_handler);
	}
	
	this.process_toggles = function() {
		var toggle_elements = $('[data-toggle-id]');
		
		// get array of toggle ids
		var id_list = {}
		toggle_elements.each(function(index, element) {
			var element = $(element);
			var cur_id = element.attr('data-toggle-id');
			id_list[cur_id] = 1;
		});
		
		for(var id in id_list) {			
			self.process_toggle_paras(id);
		}
	}
	
	this.process_toggle_paras = function(id) {
		if (id == '') return;
	
		// get toggle paragraphs by specified id
		var all_paras = $('[data-toggle-id=' + id + ']');
		
		// a toggle must include at least two paragraphs
		if (all_paras.length < 2) return;
		
		// use first paragraph as toggle title
		var title_box = all_paras.filter(':first');
		
		// use other paragraphs as toggle content
		var toggle_paras = all_paras.not(':first');
		
		// prepare title box
		self.prepare_title_box(id, title_box);
		
		// prepare toggle box
		self.prepare_toggle_box(id, toggle_paras);	
	}

	// HINT: use a table for toggle title for compatibility with old IE versions
	this.prepare_title_box = function(id, title_box) {
		// get title box elements
		var title_elements = title_box.contents();
				
		// insert table into the title paragraph
		var table = $('<table />');
		table.appendTo(title_box);
		
		// insert table row
		var tr = $('<tr />');
		tr.appendTo(table);
		
		// insert cell for toggle icon
		var icon_cell = $('<td class="icon_cell"></td>');
		icon_cell.appendTo(tr);

		// insert cell for toggle title
		var title_cell = $('<td class="title_cell"></td>');
		title_cell.appendTo(tr);
				
		// insert toggle icon
		var toggle_icon = $('<div></div>');
		toggle_icon.appendTo(icon_cell);
	
		// insert toggle link for title
		var toggle_link = $('<span></span>');
		toggle_link.appendTo(title_cell);
		
		// move title elements into the toggle link
		title_elements.detach();
		title_elements.appendTo(toggle_link);

		// update attributes
		toggle_icon.addClass('toggle_link');
		toggle_icon.addClass('toggle_icon');
		
		toggle_link.attr('class', 'toggle_link');
		
		title_box.attr('class', 'toggle_title');
		title_box.attr('data-toggle-id', id);
		title_box.addClass('is_closed');	
	}	
	
	this.prepare_toggle_box = function(id, toggle_paras) {
		// insert toggle box for content
		var toggle_box = $('<div></div>');
		toggle_box.insertBefore(toggle_paras.filter(':first'));	

		// move content paragraphs to toggle box
		toggle_paras.detach();
		toggle_paras.appendTo(toggle_box);

		// update attributes
		toggle_paras.removeAttr('data-toggle-id');
		toggle_box.attr('data-toggle-id', id);
		toggle_box.addClass('toggle_box');
		toggle_box.addClass('is_closed');
	}

	this.toggle_link_click_handler = function() {
		// get toggle link that has been clicked
		var toggle_link = $(this);
		
		// get parent title box
		var title_box = toggle_link.parents('.toggle_title:first');
		if (title_box.length < 1) return;
		
		// get toggle id
		var id = title_box.attr('data-toggle-id');
		
		// open/close toggle box by id
		self.open_close_toggle_by_id(id);
				
		return false;
	}
	
	this.open_close_toggle_by_id = function(id) {
		if (!id) return;
		
		// find title box by id
		var title_box = $('.toggle_title[data-toggle-id=' + id + ']');
		if (title_box.length <= 0) return; // nothing to do
		
		// find toggle box by id
		var toggle_box = $('.toggle_box[data-toggle-id=' + id + ']');
		if (toggle_box.length <= 0) return; // nothing to do
		
		// set animation speed
		var slide_speed = 200;
		
		// open toggle box
		if (toggle_box.hasClass('is_closed')) {
			toggle_box.slideDown(
				slide_speed,
				self.toggle_box_on_opened(title_box, toggle_box)
			);
		// close toggle box
		} else {
			toggle_box.slideUp(
				slide_speed,
				self.toggle_box_on_closed(title_box, toggle_box)
			);
		}		
	}
		
	// this event occurs after a toggle box has been opened
	this.toggle_box_on_opened = function(title_box, toggle_box) {
		return function() {
			// set attributes for "opened" state
			title_box.removeClass('is_closed');
			toggle_box.removeClass('is_closed');
			
			// update text for "toggle all" links
			self.update_toggle_all_link_text();
			
			self.update_document();
		}
	}
	
	// this event occurs after a toggle box has been closed
	this.toggle_box_on_closed = function(title_box, toggle_box) {
		return function() {
			// set attributes for "closed" state
			title_box.addClass('is_closed');
			toggle_box.addClass('is_closed');
			
			// update text for "toggle all" links
			self.update_toggle_all_link_text();
			
			self.update_document();
		}
	}
	
	this.toggle_all_click_handler = function() {
		var toggle_title = $('.toggle_title');
		if (toggle_title.length <= 0) return; // nothing to do
	
		// find at least one open toggle
		var have_open = toggle_title.not('.is_closed').length > 0;
		
		// open/hide all toggles
		toggle_title.each(function(index, element) {
			// get current toggle title
			var cur_title = $(element);
						
			// get toggle id
			var id = cur_title.attr('data-toggle-id');

			// is current toggle closed?
			var is_closed = cur_title.hasClass('is_closed');
			
			// close all toggles
			if (have_open) {
				if (!is_closed) self.open_close_toggle_by_id(id);
			} else {
			// open all toggles
				if (is_closed) self.open_close_toggle_by_id(id);
			}
		});
	}
	
	this.update_toggle_all_link_text = function() {
		// find at least one open toggle
		var have_open = $('.toggle_title').not('.is_closed').length > 0;
		
		// process "toggle all" links
		$('.toggle_all').each(function(index, element) {
			var cur_link = $(element);

			// get texts for this "toggle all" link
			var show_all_text = cur_link.attr('data-show-all-text');
			var hide_all_text = cur_link.attr('data-hide-all-text');			
						
			// both texts must be specified
			if (show_all_text && hide_all_text) {
				// set "hide all" text
				if (have_open) {
					cur_link.text(hide_all_text);
				// set "show all" text
				} else {
					cur_link.text(show_all_text);
				}
			}
		});
	}
	
	// for compatibility with non-scrolling area
	this.update_document = function() {
		$(window).trigger('resize');
	}
}

$(document).ready(function() {
	var toggle_obj = new toggle();
	toggle_obj.init();
});